import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
import os
import sys

# create_python が存在するディレクトリへのパス
pilot_generator_path = os.path.abspath(os.path.join(os.path.dirname(__file__), r"{{PILOT_GENERATOR_PATH}}"))
if pilot_generator_path not in sys.path:
    sys.path.insert(0, pilot_generator_path)

from pilot.create.pilot_generator import PilotGenerator

if __name__ == "__main__":
    print("========================================")
    print("        add_job ツール実行          ")
    print("========================================")

    # Job追加のパラメーター (必要に応じて変更・または引数化してください)
    job_no = '020'
    base_class_type = 'Job'
    #base_class_type = 'prompt'
    #base_class_type = 'generateText'
    #base_class_type = 'generateJson'
    
    # properties_file の相対パスが指定されている
    properties_file_rel = '{{PROPERTIES_FILE_REL_PATH}}'
    properties_file = os.path.abspath(os.path.join(os.path.dirname(__file__), properties_file_rel))
    
    try:
        print(f"[{properties_file_rel}] を元に Job {job_no} を追加します...")
        
        # インスタンスから add_job をcall
        generator = PilotGenerator()
        generator.add_job(
            properties_file=properties_file,
            job_no=job_no,
            base_class_type=base_class_type
        )
        print("Jobの追加処理が完了しました！\\n")
        
    except Exception as e:
        print("-" * 40)
        print(f"add_job エラーが発生しました: {e}")
""").strip()



import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
from {{JOB_PACKAGE}} import Job


class Job_000(Job):
    # __init__ は省略。BaseJobのものを継承。
    def run(self):
        self.copy_current_file_to_next_step()
        self.create_next_step_todo_trg_file()
        super().run()
""").strip()



import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
from pilot.unit.impl.base_unit import BaseUnit
from pilot.job.impl.base_job import BaseJob
from {{GYOMU_JOB_PACKAGE_000}} import Job_000
from {{GYOMU_JOB_PACKAGE_010}} import Job_010
from {{GYOMU_JOB_PACKAGE_011}} import Job_011

class Unit(BaseUnit):

    def run(self, index: int = 0):
        print(f'unit: {self.config_dto.steps[index]}')
        super().run(index)

    def _init_job(self, current_step):
        # 重要  stepe毎に　Jobを切り替える
        stepsList = self.config_dto.steps
        if current_step == stepsList[0]:
            return Job_000()
        elif current_step == stepsList[1]:
            return Job_010()
        elif current_step == stepsList[2]:
            return Job_011()
        else:
            print("不明なステップ")
        return BaseJob()
""").strip()



import threading
import time
from typing import Dict, Any, Optional
import requests
import json

from pilot.generater.generation_config import GenerationConfigDTO


class MattermostAISingleton:
    _instance: Optional['MattermostAISingleton'] = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super(MattermostAISingleton, cls).__new__(cls)
                    cls._instance._initialized = False
        return cls._instance

    def __init__(self,
                 model_name: str = "gpt-4",
                 api_key: str = "123456",
                 base_url: str = "http://127.0.0.1:8080"):

        if not self._initialized:
            with self._lock:
                if not self._initialized:
                    self.model_name = model_name
                    self.api_key = api_key
                    # 疎通コードに合わせベースURLを保持
                    self.base_url = base_url.rstrip('/')

                    self._session = requests.Session()
                    # 疎通OK版のヘッダー構成を完全に再現
                    self._session.headers.update({
                        "Authorization": f"Bearer {self.api_key}",
                        "Content-Type": "application/json"
                    })

                    self._initialized = True

    def generate_content(
            self,
            prompt: str,
            gen_config: Optional[GenerationConfigDTO] = None,
            stream: Optional[bool] = None,
            **kwargs
    ) -> Dict[str, Any]:
        """単発呼び出しモード"""
        try:
            if gen_config is None:
                gen_config = GenerationConfigDTO()

            # メッセージの組み立て (システム命令があれば先頭に追加)
            messages = []
            if gen_config.system_instruction:
                messages.append({"role": "system", "content": gen_config.system_instruction})
            messages.append({"role": "user", "content": prompt})

            # 疎通版の payload 構造をベースに gen_config を反映
            payload = {
                "model": self.model_name,
                "messages": messages,
                "stream": False,
                "temperature": gen_config.temperature if gen_config.temperature is not None else 0.7
            }

            # max_tokens が設定されている場合のみ追加
            if gen_config.max_output_tokens:
                payload["max_tokens"] = gen_config.max_output_tokens

            # 疎通版に合わせ timeout はデフォルト 120。gen_config 指定があれば優先。
            current_timeout = gen_config.timeout if gen_config.timeout else 120

            resp = self._session.post(
                f"{self.base_url}/chat/completions",
                json=payload,
                timeout=current_timeout
            )
            resp.raise_for_status()
            result = resp.json()
            content = result['choices'][0]['message']['content']

            data = json.loads(content)
            json.dumps(data, indent=4, ensure_ascii=False)
            #print(formatted_json)
            #print(result)
            #resultJson = result.json()

            # 疎通版のデータ抽出パス: result['choices'][0]['message']['content']
            content = data['respMessage']

            return {
                "prompt": prompt,
                "response": self._remove_code_fence(content),
                "success": True,
                "error": None
            }

        except Exception as e:
            return {
                "prompt": prompt,
                "response": None,
                "success": False,
                "error": str(e)
            }

    def start_chat(self):
        return _MattermostChatSession(self)

    def count_tokens(self, text: str) -> int:
        return len(text) // 4

    def _remove_code_fence(self, text: str) -> str:
        if not text: return ""
        lines = text.splitlines()
        if lines and lines[0].startswith("```"):
            lines = lines[1:]
        if lines and lines[-1].startswith("```"):
            lines = lines[:-1]
        return "\n".join(lines)

    @classmethod
    def get_instance(cls, **kwargs) -> 'MattermostAISingleton':
        return cls(**kwargs)


class _MattermostChatSession:
    """チャットセッション (継続会話)"""

    def __init__(self, client: MattermostAISingleton):
        self._client = client
        self._messages = []

    def send_message(self, message: str, gen_config: Optional[GenerationConfigDTO] = None):
        if gen_config is None:
            gen_config = GenerationConfigDTO()

        # システムプロンプト対応
        if not self._messages and gen_config.system_instruction:
            self._messages.append({"role": "system", "content": gen_config.system_instruction})

        self._messages.append({"role": "user", "content": message})

        url = f"{self._client.base_url}/chat/completions"

        payload = {
            "model": self._client.model_name,
            "messages": self._messages,
            "stream": False,  # セッションモードは基本的に同期
            "temperature": gen_config.temperature if gen_config.temperature is not None else 0.7
        }

        if gen_config.max_output_tokens:
            payload["max_tokens"] = gen_config.max_output_tokens

        resp = self._client._session.post(
            url,
            json=payload,
            timeout=gen_config.timeout if gen_config.timeout else 120
        )
        resp.raise_for_status()
        data = resp.json()

        reply = data['choices'][0]['message']['content']
        self._messages.append({"role": "assistant", "content": reply})

        class _Resp:
            def __init__(self, text):
                self.text = text

        return _Resp(self._client._remove_code_fence(reply))


import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
work_space={{WORK_SPACE}}

project={{PROJECT_NAME}}
controller_file={{CONTROLLER_FILE}}
steps={{STEP_NAME}},step_end
runsteps={{STEP_NAME}}
multisteps = step_011
threads=4
""").strip()



import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
import os.path

from {{JOB_PACKAGE}} import Job
from pilot.job.base.generate.generatePromptBaseJob import GeneratePromptBaseJob
from pilot.job.base.generate.generateTextBaseJob import GenerateTextBaseJob

class {{JOB_NAME}}(Job):

    def run(self):

        super().run()
""").strip()



import textwrap

class create_file():
    detail_prompt: str = textwrap.dedent("""\
import time
import os
import sys

from {{CONTROLLER_PACKAGE}} import Controller


# 現在のファイルのディレクトリ を基準に src フォルダのパスを取得し、sys.path の先頭に追加
project_root = os.path.dirname(os.path.abspath(__file__))
src_path = os.path.join(project_root, 'src')
if src_path not in sys.path:
    sys.path.insert(0, src_path)


if __name__ == '__main__':
    start = time.time()
    CobolController = Controller()
    CobolController.run("{{PYTHON_CONFIG_PROPERTIES}}")
    end = time.time()
    print(f"処理時間: {end - start:.2f}秒")
""").strip()
